@@ -3,64 +3,25 @@ import numpy as np
 import math
 from . import common_functions as c_f
 
-
+# input must be 2D
 def logsumexp(x, keep_mask=None, add_one=True, dim=1):
-    max_vals, _ = torch.max(x, dim=dim, keepdim=True)
-    inside_exp = x - max_vals
-    exp = torch.exp(inside_exp)
     if keep_mask is not None:
-        exp = exp*keep_mask
-    inside_log = torch.sum(exp, dim=dim, keepdim=True)
-    if add_one: 
-        inside_log = inside_log + torch.exp(-max_vals)
-    else:
-        # add one only if necessary
-        inside_log[inside_log==0] = torch.exp(-max_vals[inside_log==0])
-    return torch.log(inside_log) + max_vals
-
-def sim_mat(x, y=None):
-    """
-    returns a matrix where entry (i,j) is the dot product of x[i] and x[j]
-    """
-    if y is None:
-        y = x
-    return torch.matmul(x, y.t())
+        x = x.masked_fill(~keep_mask, c_f.neg_inf(x.dtype))
+    if add_one:
+        zeros = torch.zeros(x.size(dim-1), dtype=x.dtype, device=x.device).unsqueeze(dim)
+        x = torch.cat([x,zeros], dim=dim)
+    
+    output = torch.logsumexp(x, dim=dim, keepdim=True)
+    if keep_mask is not None:
+        output = output.masked_fill(~torch.any(keep_mask, dim=dim, keepdim=True), 0)
+    return output
 
 
-# https://discuss.pytorch.org/t/efficient-distance-matrix-computation/9065/7
-def dist_mat(x, y=None, eps=1e-16, squared=False):
-    """
-    Input: x is a Nxd matrix
-           y is an optional Mxd matirx
-    Output: dist is a NxM matrix where dist[i,j]
-    is the square norm between x[i,:] and y[j,:]
-            if y is not given then use 'y=x'.
-    i.e. dist[i,j] = ||x[i,:]-y[j,:]||
-    """
-    x_norm = (x ** 2).sum(1).view(-1, 1)
-    if y is not None:
-        y_t = torch.transpose(y, 0, 1)
-        y_norm = (y ** 2).sum(1).view(1, -1)
-    else:
-        y_t = torch.transpose(x, 0, 1)
-        y_norm = x_norm.view(1, -1)
+def meshgrid_from_sizes(x, y, dim=0):
+    a = torch.arange(x.size(dim)).to(x.device)
+    b = torch.arange(y.size(dim)).to(y.device)
+    return torch.meshgrid(a,b)
 
-    dist = x_norm + y_norm - 2.0 * torch.mm(x, y_t)
-    # Ensure diagonal is zero if x=y
-    if y is None:
-        dist = dist - torch.diag(dist.diag())
-    dist = torch.clamp(dist, 0.0, np.inf)
-    if not squared:
-        mask = (dist == 0).float()
-        dist = dist + mask * eps
-        dist = torch.sqrt(dist)
-        dist = dist * (1.0 - mask)
-    return dist
-
-def get_pairwise_mat(x, y, use_similarity, squared):
-    if x is y:
-        y = None
-    return sim_mat(x, y=y) if use_similarity else dist_mat(x, y=y, squared=squared)
 
 def get_all_pairs_indices(labels, ref_labels=None):
     """
@@ -75,11 +36,9 @@ def get_all_pairs_indices(labels, ref_labels=None):
     matches = (labels1 == labels2).byte()
     diffs = matches ^ 1
     if ref_labels is labels:
-        matches -= torch.eye(matches.size(0)).byte().to(labels.device)
-    a1_idx = matches.nonzero()[:, 0].flatten()
-    p_idx = matches.nonzero()[:, 1].flatten()
-    a2_idx = diffs.nonzero()[:, 0].flatten()
-    n_idx = diffs.nonzero()[:, 1].flatten()
+        matches.fill_diagonal_(0)
+    a1_idx, p_idx = torch.where(matches)
+    a2_idx, n_idx = torch.where(diffs)
     return a1_idx, p_idx, a2_idx, n_idx
 
 
@@ -122,26 +81,28 @@ def get_all_triplets_indices(labels, ref_labels=None):
     matches = (labels1 == labels2).byte()
     diffs = matches ^ 1
     if ref_labels is labels:
-        matches -= torch.eye(matches.size(0)).byte().to(labels.device)
+        matches.fill_diagonal_(0)
     triplets = matches.unsqueeze(2)*diffs.unsqueeze(1)
-    a_idx = triplets.nonzero()[:, 0].flatten()
-    p_idx = triplets.nonzero()[:, 1].flatten()
-    n_idx = triplets.nonzero()[:, 2].flatten()
-    return a_idx, p_idx, n_idx
+    return torch.where(triplets)
 
 
 
 # sample triplets, with a weighted distribution if weights is specified.
 def get_random_triplet_indices(labels, ref_labels=None, t_per_anchor=None, weights=None):
     a_idx, p_idx, n_idx = [], [], []
+    labels_device = labels.device
+    ref_labels = labels if ref_labels is None else ref_labels
+    ref_labels_is_labels = ref_labels is labels
     labels = labels.cpu().numpy()
-    ref_labels = labels if ref_labels is None else ref_labels.cpu().numpy()
+    ref_labels = ref_labels.cpu().numpy()
     batch_size = ref_labels.shape[0]
-    label_count = dict(zip(*np.unique(ref_labels, return_counts=True)))
     indices = np.arange(batch_size)
     for i, label in enumerate(labels):
-        curr_label_count = label_count[label]
-        if ref_labels is labels: curr_label_count -= 1
+        all_pos_pair_mask = ref_labels == label
+        if ref_labels_is_labels:
+            all_pos_pair_mask &= indices != i
+        all_pos_pair_idx = np.where(all_pos_pair_mask)[0]
+        curr_label_count = len(all_pos_pair_idx)
         if curr_label_count == 0:
             continue
         k = curr_label_count if t_per_anchor is None else t_per_anchor
@@ -153,14 +114,13 @@ def get_random_triplet_indices(labels, ref_labels=None, t_per_anchor=None, weigh
             n_idx += c_f.NUMPY_RANDOM.choice(possible_n_idx, k).tolist()
 
         a_idx.extend([i] * k)
-        curr_p_idx = c_f.safe_random_choice(np.where((ref_labels == label) & (indices != i))[0], k)
+        curr_p_idx = c_f.safe_random_choice(all_pos_pair_idx, k)
         p_idx.extend(curr_p_idx.tolist())
 
-    return (
-        torch.LongTensor(a_idx),
-        torch.LongTensor(p_idx),
-        torch.LongTensor(n_idx),
-    )
+    a_idx = torch.LongTensor(a_idx).to(labels_device)
+    p_idx = torch.LongTensor(p_idx).to(labels_device)
+    n_idx = torch.LongTensor(n_idx).to(labels_device)
+    return a_idx, p_idx, n_idx
 
 
 def repeat_to_match_size(smaller_set, larger_size, smaller_size):
@@ -199,8 +159,8 @@ def convert_to_triplets(indices_tuple, labels, t_per_anchor=100):
         if len(a1) == 0 or len(a2) == 0:
             return empty_output
         for i in range(len(labels)):
-            pos_idx = (a1 == i).nonzero().flatten()
-            neg_idx = (a2 == i).nonzero().flatten()
+            pos_idx = torch.where(a1 == i)[0]
+            neg_idx = torch.where(a2 == i)[0]
             if len(pos_idx) > 0 and len(neg_idx) > 0:
                 p_idx = p[pos_idx]
                 n_idx = n[neg_idx]
@@ -219,15 +179,17 @@ def convert_to_triplets(indices_tuple, labels, t_per_anchor=100):
 
 
 
-def convert_to_weights(indices_tuple, labels):
+def convert_to_weights(indices_tuple, labels, dtype):
     """
     Returns a weight for each batch element, based on
     how many times they appear in indices_tuple.
     """
-    weights = torch.zeros_like(labels).float()
+    weights = torch.zeros_like(labels).type(dtype)
     if indices_tuple is None:
         return weights + 1
+    if all(len(x) == 0 for x in indices_tuple):
+        return weights + 1 
     indices, counts = torch.unique(torch.cat(indices_tuple, dim=0), return_counts=True)
-    counts = (counts.float() / torch.sum(counts))
+    counts = (counts.type(dtype) / torch.sum(counts))
     weights[indices] = counts / torch.max(counts)
     return weights
